%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  CES_Simulate.m
%%%%  Simulate a network economy   
%%%%  with an exogenous network and CES production functions
%%%%
%%%%  March 20, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%


%%Output: Computed Log GDP
%%Inputs:
  % 1. N, the number of industries
  % 2. cost_io, the NxN cost-based input-output matrix
  % 3. log_tfp, an Nx1 vector of log_productivities for each industries
  % 4. markups, an Nx1 vector of markups for each industry
  % 5. sigma, the elasticity of substitution
  % 6. beta, an Nx1 vector of consumption shares for each industry

function [log_gdp ]=  CES_Simulate(N,cost_io,log_tfp,markups,sigma,beta)

%% Add Libraries to the path
addpath(genpath('./Lib'))

%Compute TFP from log_tfp
A = exp(log_tfp);


%% Compute Hicks-Neutral CES prices 

%%We use the fact that the cost function is given by 

%%  K_i = 1/A_i (  (1-\sum_{j \in S_i} \alpha_{ij})^{\sigma} +\sum_{j\in S_i} \alpha_{ij}^{\sigma} P_j^{1-\sigma})^{1/(1-sigma)}
%%  P_i = (1+\mu_i) K_i

%% To solve for P_i, note that we can write this as a linear system in terms of the variables Q_1,...,Q_n
%% where Q_i = P_i^{1-\sigma}

%%We can then solve for Q = (I-M)^{-1} R where
   
   %% M_{ij} =  ((1+mu_i/A_i)))^(1-sigma) alpha_{ij}^{\sigma}
   %% R_i = ((1+mu_i)/A_i)^(1-sigma) * (1-\sum_{j \in S_i} \alpha_{ij} )^{\sigma}



R  = zeros(N,1);
M = zeros(N,N);

for i=1:N
   R(i) = ((1+markups(i))/ A(i))^(1-sigma) * (1-sum(cost_io(i,:)))^(sigma) ;
end

for i=1:N
    for j=1:N
        M(i,j) =   ((1+markups(i)/A(i)))^(1-sigma)* cost_io(i,j)^(sigma);
    end
end

Q = (eye(N)-M)\R;
new_P = Q.^(1/(1-sigma));

%Use Shepard's Lemma to derive quantities
revenue_io = zeros(N,N);
for i=1:N
    for j=1:N
        revenue_io(i,j) = cost_io(i,j)^(sigma) * A(i)^(sigma-1) * (1/(1+markups(i)))^(sigma) * Q(j) / Q(i);
    end
end

%Use equation (D5) to compute Domar weights
revenue_domar = beta'/(eye(N)-revenue_io);

%Use equation (D2) to compute nominal GDP
nominal_gdp = (1/(1-sum(revenue_domar* (markups./(1+markups)))));
deflator = beta' * log(new_P);
log_gdp = log(nominal_gdp)-deflator;