%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% ComputeCounterfactuals.m
% Pablo Azar
% August 26th, 2018
% pazar@mit.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This file takes as inputs
%    1. An SIC Data file (SICData.mat) containing 1987 SIC level Output, Value Added, and
%    TFP for 488 industries (non-manufacturing industries are aggregated at
%    more than SIC level so we can use BEA data on TFP. Manufacturing
%    industries are reported at the 4-digit SIC level and relevant
%    variables are obtained from the NBER-CES Dataset.

%    These variables are reported for the years 1987,1992,1997,2002 and
%    2007.

%    2. A file (SIC_alphas.mat) containing the harmonized input-output matrices from
%    Acemoglu, Autor and Patterson (2017) for the years 1987,1992,1997 and
%    2002.
% 
%    3. A file (Counterfactuals.mat) containing the predicted changes in
%    industry log-productivity when regressing  \Delta \log_i TFP(t)  on the
%    Jaccard Dummy  J_{i,20}(t).
 
%    If the regression model is 

       % \Delta \log_i TFP(t) = \beta J_{i,20}(t) + \gamma_i + \nu_t + \epsilon_{i,t}
       %    the counterfactual is computed as  
                
                %  \hat{\Delta \log_i TFP(t) } = \beta J_{i,20}(t)
 

%% 1. Load The Files
clear
load('../../ProcessedData/Matlab/SICData.mat')
load('../../ProcessedData/Matlab/SIC_alphas.mat')
load('../../ProcessedData/Matlab/Counterfactuals.mat')

%% 2. Initialize the Normalized Variables

% Normalized Input Output Table ( IO table divided by Output)
normalized_alpha = zeros(488,488,5);

% Normalized Value Added Vector ( Value Added divided by GDP)
normalized_beta = zeros(488,5);

%We have the years 1987-2007 in our sample, with 5 year intervlas
years = 1987:5:2007;
TFP_matrix = zeros(488,5);

% Initialize normalized input-output matrices for first (1987) and last
% year (2007)
initial_alpha = zeros(488,488);
final_alpha = zeros(488,488);

initial_value_added = zeros(488,1);
initial_TFP = zeros(488,1);

%%Create a year index variable
year_idx=0;

%% For every year...
for year=min(years):5:max(years)
   
    %... increase the year index counter
   year_idx = year_idx + 1;
    %... get the input output matrix for that year
   CC = alpha(:,:,year_idx);
   %... set the diagonal entries to zero
   for i=1:488
       CC(i,i)=0;
   end
   %... get the output, tfp and value added for that year
   partial_table = SICData(SICData.year==year,:);
   partial_table=sortrows(partial_table,{'sic_down'});
   
   %... normalize the input-output table so that each alpha_{i,j} entry is
   %    divided by output_i
   for i=1:488
       for j=1:488
           CC(i,j) = CC(i,j) / partial_table.vship(i);
       end
   end
   
   %... set the normalized input-output matrix
   normalized_alpha(:,:,year_idx)=CC;
   
   % ... normalize the value addded vector so it adds up to 1
   normalized_beta(:,year_idx)= partial_table.vadd./sum(partial_table.vadd);

   % ... initialize the TFP vector for the given year
   TFP_matrix(:,year_idx) = partial_table.tfp5;
end



%% Compute the change in IO matrix from 
alpha0 = normalized_alpha(:,:,1);
alpha1 = normalized_alpha(:,:,5);


%% Compute the average beta vector across years
mean_beta = mean(normalized_beta,2);





%%Now compute the counterfactual numbers for Table 2: How much of the
%%change in macro TFP is due to industries adopting new combinations?

%%Load the fixed effect predictions from Stata
ImportFixedEffects


%%Iterate over filters, right hand variables (Jaccard 10th percentile, 20th
%%percentile and 30th percentile dummies), and whether the regressions are
%%weighted or not
filters = {'all','manuf','computer'};
righthands = {'dummy_10','dummy_20','dummy_30'};
weighted = {'','weighted'}

%%Also iterate over whether we run the regressions only for 1997 or not
for after_1997=1:2
    for filter=1:3
        for righthand=1:3
            for weights=1:2
              
            %Initialize a little table with one row and 3 columns. This is
            %where we are going to store the counterfactuals for this batch
            results_table = zeros(1,3);
            
            %%Give it a descriptive filename
            filename = strcat('../../ProcessedData/CSV/Counterfactuals_',filters(filter),'_',righthands(righthand),'_',string(after_1997),'_',weighted(weights),'.csv');
            
            %3 types of regressions:
               % a. time fixed effects only
               % b. time and industry
               % c. time and industry, and lagged TFP
            for regression_type=1:3
                variable_name = ['txb_',filters(filter),'_',righthands(righthand),'_'];
                if weights==2
                    variable_name = [variable_name,'w'];
                end
                
                
                if regression_type==1
                    variable_name = [variable_name,'t'];
                end
                if regression_type==2
                    variable_name = [variable_name,'ts'];
                end
                if regression_type==3
                    variable_name = [variable_name,'tsd'];
                end
                
                if after_1997 == 2
                    variable_name = [variable_name,'_7'];
                end
                strjoin(variable_name,'')
                %Have to annualize since the counterfactuals we will get
                %are over a 20 year period
                exponent = 20;
                
                %%Set the initial period. Mostly 1, but equal to 2 when
                %%using lagged TFP (can't use 1987 as the initial year
                %%because we don't have 1982 TFP), and equal to 3 when we
                %%start with 1997 data.
                initial_period = 1;
                beta = mean_beta;
                
                %Vector change in industry TFP only comes from Stata
                %predictions
                delta_tfp = FixedEffects(:,strjoin(variable_name,'')).Variables;
                if regression_type==3 
                    exponent = exponent - 5;
                    initial_period = initial_period + 1;
                end
                if after_1997 >= 2
                    exponent = 5;
                    initial_period = 3;
                end
             
                %Set the initial input-output matrix
                initial_alpha = normalized_alpha(:,:,initial_period);
        
        
                %Compute the counterfactual if we kept initial_alpha fixed,
                %but TFP changed as predicted by Stata
                results_table(regression_type) = beta' *( (eye(488)-initial_alpha) \delta_tfp)/exponent; 
            end
            %write it in a file
            csvwrite(filename,results_table)
       end
       end
    end
end
