%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% ComputeGDP.m
% 
% March 20th, 2019
% Last Updated: May 12th, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% A function that computes real gdp, nominal gdp and domar weights of a Cobb-Douglas IO economy with markups

%%INPUTS
%N: The number of industries

%tfp:  a vector of log-tfp by industry

%markups: a vector of markups by industry

%alpha: the cost_based input-output matrix 
    %   alpha(i,j) = (P_j X_{ij}) / (\sum_{j} P_j X_{ij} + L_i) 
    
%beta: consumption shares by industry
    % beta(i) = P_i C_i / (\sum_{i}  P_i C_i) 

%%OUTPUTS

%gdp: (this measures log(gdp)

%cost_domar: beta' (I-alpha)^{-1}, the "cost-based domar weights". Useful
%for aggregation

%P : a vector of prices by industry

%X: the matrix of intermediate input demands

%Y: a vector of output by industry

%W: wages

% value_added: a vector of value added by industry

function [gdp,cost_domar,domar,P,X,Y,W,value_added] = ComputeGDP(N,tfp,markups,alpha,beta)
    
    %1.  Compute the revenue_based input_output matrix using equation (D3)
    for i=1:N
        for j=1:N
            alpha_hat(i,j) = alpha(i,j) / (1+markups(i));
        end
    end
    
   
    %2. Compute the Domar Weights using equation (D5)
    domar = beta' / (eye(N)-alpha_hat);
    
    %3. Compute log-GDP using equation (D6)
    gdp  = (beta'/( eye(N)- alpha)) * (tfp-log(1+markups)) - log(1- sum(domar) + sum( domar' ./(1+markups)));
    
    
    %4. Compute the decomposition of output P_i Y_i = Value Added_i + \sum_{j} X_{ij} P_j 
    
    %%4.a Compute prices, revenue, and wages
    log_prices = -(eye(N)-alpha)\(tfp-log(1+markups));
    P = exp(log_prices);
    PY = exp(gdp) * domar;
    W = exp( (beta'/( eye(N)- alpha)) * (tfp-log(1+markups)));
    P = P * W;
    
    %%4.b Compute intermediate inputs 
    X = zeros(N,N);   
    Y = PY./P;
    for i=1:N
        for j=1:N
            X(i,j) = alpha_hat(i,j) * PY(i) / P(j);
        end
    end

    %%4.c Compute Value added by industry
    value_added = zeros(N,1);
    for i=1:N
       value_added(i) = PY(i); 
       for j=1:N
           value_added(i) = value_added(i) - P(j)*X(i,j);
       end
    end
    
    
    %%5. Finally, for aggregation purposes, compute the "cost-based domar
    %%weights"  using equation (D18)
    
    
    %This is helpful for aggregation
    
    cost_domar = (beta'/( eye(N)- alpha));

    
end

