%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% ComputeSimulations.m
% 
% March 20th, 2019
% Last Updated: May 12th, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% This is the main file to Compute the simulations in Appendix D of Acemoglu and Azar (2019)
%% The file takes as inputs
  %1. The 2007 Input Output table from the BEA
  %2. A vector of value added by industry from the BEA
  %3. A vector of commodity names from the BEA
  %4. A vector of industry names from the BEA
  %5. A vector of markups by 2-digit industry from De Loecker, Eeckhout and
  %   Unger (2018)
  %6. A vector of Consumption Shares by Industry from the BEA



%%  Load The Files
clear

% 2007 input-output matrix from the BEA
load('../../ProcessedData/Matlab/InputOutput.mat')

%Vector of Value Added By Industry
load('../../ProcessedData/Matlab/ValueAdded.mat')

%Vector of Commodity Names
load('../../ProcessedData/Matlab/CommodityNames.mat')

%Vector of Industry Names
load('../../ProcessedData/Matlab/IndustryNames.mat')

%Vector of Markups by Industry
load('../../ProcessedData/Matlab/markups.mat')

%Vector of Consumption Shares by Industry
load('../../ProcessedData/Matlab/Consumption.mat')

%%Process the data
%% This includes
  %1. Making the given input-output matrix by transforming it from
  %Commodity x Industry to Commodity x Commodity
  
  %2. Removing industries which have 0 labor share
  
  %3. Computing consumption shares
[revenue_io,labor_share,gdp_shares,N,CommodityNames,IndustryNames,ValueAdded,Consumption]=ProcessData(CommodityNames,IndustryNames,InputOutput,ValueAdded,Consumption);

%%Create vectors of 3-digit and 2-digit commodity codes 
three_digit_commodity_names = categorical(zeros(N,1));
two_digit_commodity_names = categorical(zeros(N,1));

 for i=1:N
    three_digit_commodity_names(i) = NAICS6ToNAICS3(CommodityNames(i)); 
    two_digit_commodity_names(i) = NAICS6ToNAICS2(CommodityNames(i));
 end

 %%Create vector of 6-digit industry markups from vector of 2-digit
 %%industry markups
long_markups = zeros(N,1); 
n = length(markups);
for i=1:n
    idx = find(categorical(markups(i,1)) == two_digit_commodity_names);
    long_markups(idx) = markups(i,2)/100;
end
markups= long_markups;
markups(markups<0)=0;

%%Obtain cost based and revenue based input-output matrix
cost_io = revenue_io ./ (sum(revenue_io,2) + labor_share);
revenue_io = cost_io ./ (1+markups);


%Set the random seed for replicability
rng(11111);

%%Calibration Target: Real 2007 GPD in millions of dollars
real_gdp = sum(ValueAdded);



% To calibrate the parameter mu, uncomment the following line
% In this code, we set the parameter mu = 0.24821 * log(real_gdp)
% which, in our baseline model, approximates log(real_gdp) to within 4
% decimal digits

%mu =   EN_Calibrate(N,cost_io,gdp_shares,markups,log(real_gdp));
mu = 0.24821* log(real_gdp);


%% Compute the b0,b parameters that are coherent with the observed network and compute the imputed input-output matrix
%%    
[b0,b,imputed_alpha] =  EN_Initialize(N,cost_io,markups,mu);


%%  Use the computed b,b0 and the imputed input output matrix to compute GDP and domar
%% weights if there was no shock

[gdp_no_shock,obs_alphas_no_shock,cost_domar,revenue_domar,tfp,b,decomposition_no_shock] =  EN_Simulate(N,cost_io,imputed_alpha,gdp_shares,markups,b0,b);

%%Note: the returned input-output matrix shouldn't change since we haven't
%%shocked the economy yet

assert(nnz(obs_alphas_no_shock-cost_io)==0);

%%  Shock the TFP of the electronics 3-digit SIC sector by 1 percent

electronics_industries = find(three_digit_commodity_names=="334");
b0(electronics_industries) = b0(electronics_industries)+0.01;

%%  Recompute GDP after the shock. Note now we should have a changed
%%input-output matrix
[gdp_shock,cost_io_shock,cost_domar_shock,revenue_domar_shock,tfp_shock,b,decomposition_shock] =  EN_Simulate(N,cost_io,imputed_alpha,gdp_shares,markups,b0,b);
revenue_io_shock = cost_io_shock./ (1+markups);
display(nnz(cost_io_shock-imputed_alpha));

%% For reporting, show the histograms of b_{ij} for  j observed in S_i, and b_{ij} for j \not in S_i
hist(b(cost_io > 0))
print('../../Figures/PositiveHistogram.pdf','-dpdf')


hist(b(cost_io <= 0))
print('../../Figures/NegativeHistogram.pdf','-dpdf')

%%   Perform Domar aggregation using the Domar weights from the economy without shocks

[n,domar_agg,cost_io_agg,tfp_agg,beta_agg,markups_agg] = DomarAggregate(CommodityNames,three_digit_commodity_names,cost_domar,revenue_domar,cost_io,tfp,gdp_shares,markups);


%%  Compute GDP on the aggregate economy without shocks
[gdp_agg_no_shock,cost_domar_agg_no_shock,revenue_domar_agg_no_shock,P_no_shock,X_no_shock,Y_no_shock,W_no_shock] = ComputeGDP(n,tfp_agg,markups_agg,cost_io_agg,beta_agg);


%%We now shock the TFP of the electronics (SIC 36) sectors by 1 percent. This is the same shock that we did for the
%%aggregate economy. 
aggregate_shock = zeros(n,1);
aggregate_shock(17) = 0.01;

%%  Compute GDP on the aggregate economy with shocks
[gdp_agg_shock,cost_domar_agg_shock,revenue_domar_agg_shock,P_shock,X_shock] = ComputeGDP(n,tfp_agg+aggregate_shock,markups_agg,cost_io_agg,beta_agg);



%%  Compute log gdp for the Cobb-Douglas disaggregate and aggregate eocnomies with and without shocks
observed_log_gdp = log(real_gdp)
log_gdp_disaggregate_no_shock = gdp_no_shock;
log_gdp_aggregate_no_shock = gdp_agg_no_shock;
log_gdp_disaggregate_shock = gdp_shock;
log_gdp_aggregate_shock = gdp_agg_shock;


% Compute number of edges added
number_of_edges_added = sum(sum( ((cost_io_shock>0) - (cost_io>0)) == 1))

%Now we compute log gdp for CES aggregate economies before and after the
%shock

%Note that we normalize TFP so that a_i = a_i - 1/N \sum_{i=1}^N a_i
%This helps prevent numerical issues when computing CES prices
tfp_agg = tfp_agg-mean(tfp_agg);
CES_gdp_half_no_shock = CES_Simulate(n,cost_io_agg,tfp_agg,markups_agg,1/2,beta_agg);
CES_gdp_half_shock = CES_Simulate(n,cost_io_agg,tfp_agg+aggregate_shock,markups_agg,1/2,beta_agg);
CES_gdp_two_no_shock = CES_Simulate(n,cost_io_agg,tfp_agg,markups_agg,2,beta_agg);
CES_gdp_two_shock = CES_Simulate(n,cost_io_agg,tfp_agg+aggregate_shock,markups_agg,2,beta_agg);

%%% Report Final Numbers

disaggregate_tfp_change = gdp_shock-gdp_no_shock
aggregate_tfp_change = gdp_agg_shock - gdp_agg_no_shock
CES_half_change = CES_gdp_half_shock-CES_gdp_half_no_shock
CES_two_change = CES_gdp_two_shock - CES_gdp_two_no_shock

%Compute the value added of the electronics industry
computer_value_added = sum(decomposition_shock(three_digit_commodity_names=="334"))-sum(decomposition_no_shock(three_digit_commodity_names=="334"));

%Compute the contribution of the electronics industry to growth
computer_value_added_growth  = computer_value_added / sum(decomposition_no_shock)
