%%Code that performs Domar aggregation

%% This code follows the algorithm described  in Appendix D of Acemoglu and Azar (2019). This algorithm is adapted
%% from the algorithm in Acemoglu, Ozdaglar and Tahbaz-Salehi (2017) to account for markups
%%and  with help from
%% The algorithm is based on notes by Alireza Tahbaz-Salehi for Domar Aggregtion with markups
%%

%%INPUTS:

  %1. long_sic: a vector of 6-digit SIC codes
  %2. short_sic: a vector of 3-digit SIC codes
  %3. cost_domar: a vector of disaggregated cost-based Domar weights
  %4. revenue_domar: a vector of disaggregated Domar weights
  %5. alpha: The disaggregate cost-based input-output matrix
  %6. tfp: The disaggregate vector of industry TFP
  %7. beta: The disaggregate vector of consumption shares
  %8. markups: The disaggregate vector of markups
  
%% Outputs

  %1. N: The number of aggregated sectors
  %2. domar_agg: The aggregated (revenue-based) domar weights by sector
  %3. alpha_agg: The aggregated input-output matrix
  %4. tfp_agg: The aggregated vector of sector TFP
  %5. beta_agg: The aggregated vector of consumption shares by sector
  %6. markups_agg: The aggregated vector of markups by sector

function [N,domar_agg,alpha_agg,tfp_agg,beta_agg,markups_agg] = DomarAggregate(long_sic,short_sic,cost_domar,revenue_domar,alpha,tfp,beta,markups)

%N is the number of sectors (aggregate-level)
N = length(unique(short_sic));

%n is the number of industries (disaggregate-level)
n = length(unique(long_sic));

%Initialize aggregate domar weights, shocks, consumption shares,
%input-output matrix, and markup vectors
domar_agg = zeros(N,1);
tfp_agg = zeros(N,1);
beta_agg = zeros(N,1);
alpha_agg = zeros(N,N);
markups_agg = zeros(N,1);



%%Generate  list of aggregate sector codes
unique_short_sic = unique(short_sic);


%%Iterate over all aggregate sectors
%% For each sector...
for I = 1:N
    
    %a. Get the sector code
    short_idx = unique_short_sic(I);
    
    %%b. Get the indices for industries that belong to that sector
    long_indices = (short_sic == short_idx);
    
    %%c. Aggregate consumption shares using equation (D13)
    beta_agg(I) = sum(beta(long_indices));

    
    %d. Aggregate domar weights using equation (D14)
    domar_agg(I) = sum(revenue_domar(long_indices));
    
    %e. Aggregate markups using equation (D15)
    if domar_agg(I)==0
        markups_agg(I) = 0;
    else
        markups_agg(I) = sum( revenue_domar(long_indices)) / (sum((revenue_domar(long_indices))'./ (1+markups(long_indices))))  -1;
    end
    
    %f. Aggregate the input-output matrix using equation (D16)
    for J=1:N
        short_j = unique_short_sic(J);
        j_long_indices = (short_sic == short_j);
        restricted_cost_io = alpha(long_indices,j_long_indices);
        if domar_agg(I) == 0
            alpha_agg(I,J) = 0;
        else
            alpha_agg(I,J) = (1+markups_agg(I))/domar_agg(I) * ((revenue_domar(long_indices)./(1+markups(long_indices))') * (sum(restricted_cost_io,2)) );
        end
    end
   
end

    %g. Compute the aggregate cost-based Domar weights using equation (D17)
    cost_domar_agg = beta_agg' / (eye(N)-alpha_agg) ;
    
    %h. Compute aggregate shocks using equation (D18)
for I=1:N  
    short_idx = unique_short_sic(I);
   long_indices = (short_sic == short_idx);
  
  if cost_domar_agg(I) == 0
    tfp_agg(I) = log(1+markups_agg(I));
  else
      tfp_agg(I) = 1/cost_domar_agg(I) * ( cost_domar(long_indices) * tfp(long_indices)) - 1/cost_domar_agg(I) * ( cost_domar(long_indices) * log(1+markups(long_indices))) + log(1+markups_agg(I));
  end
end
    
   

   

end

