%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  EN_Initialize.m
%%%%  Impute the parameters of a Cobb-Douglas economy  
%%%%  with endogenous network creation
%%%%
%%%%  
%%%%  March 20, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Inputs
% N : The number of industries
% initial_alpha : The input-output matrix before the shock
% markups : markups for each industry
% mu: the distribution parameter for b_{i0} and b_{ij}

%Outputs
% b_0: the imputed vector (b_{i0})_{i=1}^N
% b: the imputed matrix b_{ij}, given the observed network
% alpha: the imputed alpha_{ij}, including imputed weights



%%

function [b0,b,alpha] =  EN_Initialize(N,initial_alpha,markups,mu)



%% Add Libraries to the path
addpath(genpath('./Lib'))

%% We are using a model where a_i(S_i) = \sum_{j \in S_i} b_{ij}, and b_{ij}~ Exp(0.00001/N), where N is the number of industries
%% We normalized by N so that a shock to all (b_{ij})_{j=1}^N does not scale with N

%%Impute  unobserved alphas with the following rule
  %a. Compute the average  outdegree O_j =  \sum_{i=1}^n alpha_{ij}* 1 _{j \in S_i}
  %b. For every customer i
  %... i Assign  95 percent of the labor share (1- \sum_{j \in S_i}
  %\alpha_{ij}) to
  %each unobserved supplier, j, proportionally to O_j
alpha = initial_alpha;
row_sums = sum(initial_alpha,2);
column_sums = sum(initial_alpha,1);
entire_sum = sum(column_sums);
for i=1:N
    suppliers_i = [];
    for j=1:N
        if initial_alpha(i,j) <=0
            suppliers_i = [suppliers_i, j];
        end
    end
   
    for k=1:length(suppliers_i)
        j = suppliers_i(k);
        alpha(i,j) = 0.95 * (1-row_sums(i)) *  column_sums(j)/entire_sum; %* (1/sum(initial_alpha(i,:)<=0));
    end
        
end


%% Draw b_{i0} from a normal prior with mean mu and standard deviation 1
b0 = normrnd(mu,1,N,1);

%Draw b_{ij} from a normal prior with mean mu/N and standard deviation 1/N
b = normrnd(mu/N,1/N,N,N);

%Compute a_i(S_i)
binary_alpha = (initial_alpha > 0);
total_productivity=sum(binary_alpha .* b,2)+b0;

%% Compute log-prices  p = -(I-alpha(S))^(-1) ( a-log(1+markups))
p = - ( (eye(N)-initial_alpha) \ (total_productivity - log(1+markups)));


%%Need to do bayesian updating on b given the observed network. This
%%Algorithm is described in Appendix D of Acemoglu and Azar (2019)

%Encode the prior distribution
pd = makedist('Normal','mu',mu/N,'sigma',1/N);
my_max = pd.icdf(0.999999999999999);
my_min = pd.icdf(1-0.999999999999999);

%%Binary input output matrix
binary_alpha = (initial_alpha > 0);
binary_alpha_stacked = reshape(binary_alpha,N^2,1);

%Some helpful vectors to cut off the distribution of b_{ij}
%If (i,j) is observed, then  b_{ij} \in [p_j*alpha_{ij},+inf]. If (i,j) is
%not observed, then b_{ij} \in [-inf,p_j*alpha_{ij}].

%%Note that since p_j changes with b_{ij}, need to iterate this
%%procedure until convergence, with the thresholds determined dynamically

infinity_stacked = zeros(N^2,1);
minus_infinity_stacked = zeros(N^2,1);
for i=1:N^2
    if binary_alpha_stacked(i)
        minus_infinity_stacked(i) = 0;
        infinity_stacked(i) = Inf;
    else
        minus_infinity_stacked(i) = -Inf;
        infinity_stacked(i) = 0;
    end
end

mu_stacked = mu/N * ones(N^2,1);
sigma_stacked = 1/N * ones(N^2,1);
%%Iterate until all edges in the network are coherent with drawn b_{ij}
flag = 0;
thresholds = zeros(N^2,1);
while (flag == 0)
    
    flag = 1;


    %%These lines compute the correct left and right thresholds,
    %%respectively
    l =  minus_infinity_stacked + thresholds;
    u =  infinity_stacked + thresholds;
    
    %%Draw a tentative b_{ij} vector given the observed thresholds
    tentative_b = reshape(trandn_helper(l,u,mu_stacked,sigma_stacked),N,N);
    
    %%Compute the thresholds given the observed prices
        for i=1:N
            for j=1:N
                idx = sub2ind([N,N],i,j);
            thresholds(idx) = p(j) * alpha(i,j) ;
            
            %% (Step 4.a in the algorithm)
            %If there's a false positive, replace the existing b_{ij} with
            %%the b_{ij} ~ [-inf, p_j alpha_{ij}]
            if binary_alpha(i,j)==0
                if b(i,j) > thresholds(idx)
                    flag = 0;
                    b(i,j) =  tentative_b(i,j);
                end
            end
            
            %% (Step 4.b in the algorithm)
            %%If there's a false negative, replace the existing b_{ij} with
            %%the b_{ij} ~ [ p_j alpha_{ij},+inf]
            if binary_alpha(i,j) == 1
                if b(i,j) <= thresholds(idx)
                    flag = 0;
                    b(i,j) =tentative_b(i,j);
                end
            end
            end
        end
        
        %% (Step 4.d  in the algorithm)
        %recompute the prices and productivities before iterating
        total_productivity=sum(binary_alpha .* b,2)+b0;
        p = - ( (eye(N)-initial_alpha) \ (total_productivity - log(1+markups)));
    
        
    
        
end
        
display('Converged')


end


