%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  EN_Simulate.m
%%%%  Simulate a network economy   
%%%%  with endogenous network creation
%%%%
%%%%  
%%%%  March 20, 2019
%%%%%%%%%%%%%%%%%%%%%%%%%%

%%INPUTS

%N: The number of industries

% initial_alpha: the initial cost-based input-output matrix from the data 
 

       %%  initial_alpha(i,j) = (P_j X_{ij}) / (\sum_{j} P_j X_{ij} + L_i) 

% imputed_alpha: the cost-based input-output matrix observed from the data, with missing entries imputed by EN_Initialize

% initial_beta: The GDP shares observed in the data
  
        %% initial_beta(i) = P_i C_i / (\sum_{i=1}^N P_i C_i)

% markups: Markups from De Loecker, Eeckhout and Unger (2018) 


%%OUTPUTS:

%gdp: The computed real log-GDP

%new_alpha: The computed endogenous input-output matrix with the given b0,b

%cost_domar: The cost-based Domar weights defined in equation (D13)

%domar: The domar weights by industry d_i = P_i Y_i / GDP

%TFP: The reduced form tfp : tfp_i = b_{i0}+ \sum_{j \in S_i}  b_{ij}

%b: The matrix of b_{ij} after the network changes (for reporting purposes)

%value_added: A vector of value added by industry

%%

function [gdp,new_alpha,cost_domar,revenue_domar,tfp,b,value_added] =  EN_Simulate(N,initial_alpha,imputed_alpha,initial_beta,markups,b0,b)



%%Some accounting: keep vectors of old and new prices, to track
%%convergence.  Keep track of new input-outuput matrix
p_old = zeros(N,1);

%Binary input-output matrix
binary_alpha = (initial_alpha > 0);

%Compute initial TFP  :  tfp_i = b_{i0} + \sum_{j \in S_i} b_{ij}
tfp = sum(binary_alpha .* b,2)+b0;

%Compute the initial p_new
p_new = -(eye(N)-initial_alpha) \ ( tfp-log(1+markups));

%Initialize the endogenous IO matrix to be the one observed in the data
new_alpha = initial_alpha;

%%Repeat until convergence
while(max(abs(p_old-p_new))>0)
   %For every customer i...
   for i=1:N
       % For every potential supplier j...
       for j=1:N
           % If adopting that supplier decreases the cost function...
           if (b(i,j) > p_new(j) * imputed_alpha(i,j)) && (new_alpha(i,j)<=0)
                %Change the input-output matrix to add that supplier
                new_alpha(i,j) = imputed_alpha(i,j);
           end
       end
   end
   
   %%Keep track of the sets
   binary_alpha = (new_alpha > 0);
   %%Compute the new quantities a(S_i) = b_{i0}+ sum_{j \in S_i} b_{ij} using the
   %%new input-output matrix
   tfp = sum(binary_alpha .* b,2)+b0;
   
   %Update prices
   p_old = p_new;
   p_new = - ( (eye(N)-new_alpha) \ (tfp-log(1+markups)));
   
     
end

%%Now we compute equilibrium quantities  given prices

[gdp,cost_domar,revenue_domar,P,X,Y,W,value_added] = ComputeGDP(N,tfp,markups,new_alpha,initial_beta);


end








