/******************************
Process Data
Pablo Azar, August 25th 2018
******************************/

/*******************************************************************************************************************************************************************************
 This script takes as input the full dataset of Harmonized input-output tables at the SIC 1987 level   {alpha(t)}_{t=1972,1977,...,2007}
 It outputs a dataset that is a list of (customer,year,indegree,jaccard) where

   a. Customer is a SIC industry
   b. Year is a yera in 1972,1977,1982,1987,1992,1997,2002 or 2007
   c.  Indegree is the customer's indegree for that year
   d.  Jaccard is the customer's  Jaccard distance   1-  \frac{  |S_i(t) \cap S_i(t-1)|}{|S_i(t) \cup S_i(t-1)|}. If  S_i(t) \cup S_i(t-1) = \emptyset, then jaccard = 0.
********************************************************************************************************************************************************************************/

  
/**************************************
Load the harmonized input output tables
***************************************/

use "../../Data/Full_table.dta",clear

/**************************************
Merge with the descriptions of SIC codes
***************************************/

rename sic_down SIC
merge m:1 SIC using "../../Data/SICCodes"
drop _merge
rename SIC sic
rename sic_up SIC
rename Description description_customer
merge m:1 SIC using "../../Data/SICCodes"
drop _merge
rename SIC sic_supplier
rename Description description_supplier
replace description_supplier = sic_supplier if description_supplier == ""
replace description_customer = sic if description_customer == ""

/**************************************
Merge with the descriptions of SIC codes
***************************************/
drop if sic_supplier ==""
drop if sic == ""

//Move everything to lowercarse, and create SIC numbers for manufacturing industries (other industries are aggregated at the NIPA level and do not have numbers)
replace description_supplier = lower(description_supplier)
replace description_customer = lower(description_customer)
destring sic,force gen(sic_number)
destring sic_supplier,force gen(supplier_number)

//Generate a group for each (customer,supplier pair)
egen ij_group = group(sic sic_supplier)

//Generate a binary flag if the edge (customer,supplier) is in the network
// Below, I refer to this value as binary_alpha_{i,j,t}, indicating the presence or absence of edge (i,j) at time t
gen flag = use_value > 0

//One period is 5 years. Need to map (1972,1977,...,2007) to (1,2,3...,8) in order for stata lags and differences to make sense
gen t = (year-1972)/5+1

// Balance the panel, and set the edge flag to 0 if a (customer,supplier,year) triple was not in the original unbalanced panel
fillin ij_group t
replace flag = 0 if flag == .

//Set panel data
xtset ij_group t


/*
  Compute the Jaccard distance  d(S_{i,t},S_{i,t-1}) = 1 - |S_{i,t} \cap S_{i,t-1}|/|S_{i,t} \cup S_{i,t-1}|
*/

//Compute
  //a. d(i,j,t) =   binary_alpha(i,j,t) - binary_alpha(i,j,t-1)
  //b. m(i,j,t) =   min(binary_alpha(i,j,t),binary_alpha(i,j,t-1))
  //c. M(i,j,t) =   max(binary_alpha(i,j,t),binary_alpha(i,j,t-1))
   //Note that for two numbers  x,y \in {0,1} , we have  min(x,y) = x*y, max(x,y)  = 1 - (1-x)*(1-y)
   
gen ij_min = flag * L.flag
gen ij_max = 1 - (1-flag)*(1-L.flag)



bysort sic year: egen indegree = total(flag)
bysort sic year: egen union = total(ij_max)
bysort sic year: egen intersection = total(ij_min)
gen jaccard = 1- intersection / union
replace jaccard = 0 if union == 0
assert jaccard >=0
assert jaccard <= 1

keep sic year indegree  jaccard  
duplicates drop


save "../../ProcessedData/Stata/SetDifferencesAndIndegrees",replace

