/*
run_regressions.do

File to run the regressions for Table 1 and Tables C1-C4 in Acemoglu and Azar (2019)

This file takes as input ../../ProcessedData/Data/MergedDataset.dta
and outputs 

  a. Table files in the folder ../../Tables/, to be used by the script ../../Code/Python/writeTables.py (which produces the final tables)
  b. A file FixedEffects.csv passed to the script ../../Code/Matlab/ComputeCounterfactuals.m to compute the counterfactual effects of firms changing their suppliers on GDP


We regress  

 \Delta log TFP_{i,t} = \beta J_{i,q,t} + \gamma_i + \delta(t) + \nu \Delta log TFP_{i,t-1} +  \epsilon_{it}
 
 where
    a. i is an industry
	b. t is a hyear
    c. \Delta log TFP_{i,t} = log TFP_{i,t} - log TFP_{i,t-1}
    d. J_{i,q,t} is a dummy variable equal to 1 if industry i's Jaccard Distance \Delta_{Jaccard}(S_{i,t},S_{i,t-1})
 is above the q^{th} percentile for year t, and 0 otherwise
    e. \gamma_i is an industry fixed effect
	f. \delta(t) is a time fixed effect
  
*/


//Load the input file
use "../../ProcessedData/Stata/MergedDataset",clear


//Assign labels to the independent variables. J_{i,t,q} = 1 if industry i's Jaccard distance is above the q^{th} quartile at time t, and 0 otherwise
label variable dummy_10 "J\$_{i,10}\$"
label variable dummy_20 "J\$_{i,20}\$"
label variable dummy_30 "J\$_{i,30}\$"

//Set sic_group as the panel and t as the time variable. Sic_group is a numeric group unique to each SIC industry. t \in \{1,...,5\} represents the years
// 1987, 1992, 1997, 2002, 2007
xtset sic_group t

set more off

//Run the regressions for 
  //a. all industries
  //b. manufacturing industries
  //c. All industries excluding computers
foreach filter in all  manuf computer  {

 
// Run the regression using different Jaccard dummies (10th, 20th and 30th percentile dummies)
foreach righthand in   dummy_10 dummy_20 dummy_30 {


  //Time fixed effects
 reg delta_tfp `righthand' i.t     if `filter'==1, vce(cluster sic_group)
 
  //Generate predicted effect only from the Jaccard Dummy being `on' or `off'
  gen xb_`filter'_`righthand'_t = _b[`righthand'] * `righthand'
  
  //Output regression results to table
  outreg2 using "../../Tables/final_`filter'_`righthand'.tex",   tex(frag) replace nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)


 //Time and SIC fixed effects
 reg delta_tfp `righthand'   i.t i.sic_group   if `filter'==1,   vce(cluster sic_group)
 gen xb_`filter'_`righthand'_ts = _b[`righthand'] * `righthand'
  outreg2 using "../../Tables/final_`filter'_`righthand'.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)
 
 //Time and SIC fixed effects, and including lagged change in TFP as regressor
 reg delta_tfp `righthand'   L.delta_tfp   i.t i.sic_group   if `filter'==1,   vce(cluster sic_group)
  gen  xb_`filter'_`righthand'_tsd = _b[`righthand'] * `righthand'
  
  //Compute the necessary coefficients to compute the indirect effect of changing input sets on current TFP, where this indirect effect
  // is through lagged TFP
  
  // Predicted Change in TFP = \beta_{Jaccard} * Jaccard_t + \beta_{Lag_TFP} * \beta_{Jaccard} * Jaccard_{t-1} + (\beta_{Lag_TFP})^2 * \beta_{Jaccard} * Jaccard_{t-2} + ..
  gen  lag_`filter'_`righthand' = _b[L.delta_tfp]
  gen  x1_`filter'_`righthand' = lag_`filter'_`righthand' *  _b[`righthand'] * L.`righthand'
  gen  x2_`filter'_`righthand' =(lag_`filter'_`righthand')^2 *  _b[`righthand'] * L2.`righthand'
  gen  x3_`filter'_`righthand' = (lag_`filter'_`righthand')^3 *  _b[`righthand'] * L3.`righthand'
  gen  x4_`filter'_`righthand' = (lag_`filter'_`righthand')^4 *  _b[`righthand'] * L4.`righthand'
  
  replace x1_`filter'_`righthand' = 0 if x1_`filter'_`righthand' == .
  replace x2_`filter'_`righthand' = 0 if x2_`filter'_`righthand' == .
  replace x3_`filter'_`righthand' = 0 if x3_`filter'_`righthand' == .
  replace x4_`filter'_`righthand' = 0 if x4_`filter'_`righthand' == .

  
  replace xb_`filter'_`righthand'_tsd =  xb_`filter'_`righthand'_tsd  + x1_`filter'_`righthand' + x2_`filter'_`righthand' +  x3_`filter'_`righthand' + x4_`filter'_`righthand'


 outreg2 using "../../Tables/final_`filter'_`righthand'.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time+SIC")  label symbol(,,,)

 
 //The next three regressions repeat the above three regressions, but using value added as a weight in observatinos
 reg delta_tfp `righthand'   i.t   [weight=vadd_weight] if `filter'==1, vce(cluster sic_group)
  gen xb_`filter'_`righthand'_wt = _b[`righthand'] * `righthand'

  outreg2 using "../../Tables/final_`filter'_`righthand'_weighted.tex",   tex(frag) replace nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)


 reg delta_tfp `righthand'   i.t i.sic_group    [weight=vadd_weight] if `filter'==1,   vce(cluster sic_group)
  gen xb_`filter'_`righthand'_wts = _b[`righthand'] * `righthand'

  outreg2 using "../../Tables/final_`filter'_`righthand'_weighted.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)
 
 xtreg delta_tfp `righthand'   L.delta_tfp   i.t    [weight=vadd_weight] if `filter'==1, fe  vce(cluster sic_group)
   gen xb_`filter'_`righthand'_wtsd = _b[`righthand'] * `righthand'
  gen  lag_`filter'_`righthand'_w = _b[L.delta_tfp]
  gen  x1_`filter'_`righthand'_w = lag_`filter'_`righthand' *  _b[`righthand'] * L.`righthand'
  gen  x2_`filter'_`righthand'_w =(lag_`filter'_`righthand')^2 *  _b[`righthand'] * L2.`righthand'
  gen  x3_`filter'_`righthand'_w = (lag_`filter'_`righthand')^3 *  _b[`righthand'] * L3.`righthand'
  gen  x4_`filter'_`righthand'_w = (lag_`filter'_`righthand')^4 *  _b[`righthand'] * L4.`righthand'
  
  replace x1_`filter'_`righthand'_w = 0 if x1_`filter'_`righthand'_w == .
  replace x2_`filter'_`righthand'_w = 0 if x2_`filter'_`righthand'_w == .
  replace x3_`filter'_`righthand'_w = 0 if x3_`filter'_`righthand'_w == .
  replace x4_`filter'_`righthand'_w = 0 if x4_`filter'_`righthand'_w  == .


   replace xb_`filter'_`righthand'_wtsd =  xb_`filter'_`righthand'_wtsd + x1_`filter'_`righthand'_w + x2_`filter'_`righthand'_w +  x3_`filter'_`righthand'_w + x4_`filter'_`righthand'_w

 outreg2 using "../../Tables/final_`filter'_`righthand'_weighted.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time+SIC")  label symbol(,,,)
}
}


//The next regressions repeat the above block, but restricting the observations to be after 1997.

foreach filter in all  manuf computer  {

 

foreach righthand in  dummy_10 dummy_20 dummy_30 {


 reg delta_tfp `righthand' i.t      if (`filter'==1 & year > 1997), vce(cluster sic_group)
  gen xb_`filter'_`righthand'_t_7 = _b[`righthand'] * `righthand'
  outreg2 using "../../Tables/final_`filter'_`righthand'_1997.tex",   tex(frag) replace nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)


 reg delta_tfp `righthand'  i.t i.sic_group     if (`filter'==1 & year > 1997),   vce(cluster sic_group)
 gen xb_`filter'_`righthand'_ts_7 = _b[`righthand'] * `righthand'
  outreg2 using "../../Tables/final_`filter'_`righthand'_1997.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)
 
 reg delta_tfp  `righthand'   L.delta_tfp i.t i.sic_group     if (`filter'==1 & year > 1997),   vce(cluster sic_group)
  gen xb_`filter'_`righthand'_tsd_7 = _b[`righthand'] * `righthand'
 outreg2 using "../../Tables/final_`filter'_`righthand'_1997.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time+SIC")  label symbol(,,,)
 gen  lag_`filter'_`righthand'_7 = _b[L.delta_tfp]
  gen  x1_`filter'_`righthand'_7 = lag_`filter'_`righthand' *  _b[`righthand'] * L.`righthand'
  gen  x2_`filter'_`righthand'_7 =(lag_`filter'_`righthand')^2 *  _b[`righthand'] * L2.`righthand'
  gen  x3_`filter'_`righthand'_7 = (lag_`filter'_`righthand')^3 *  _b[`righthand'] * L3.`righthand'
  gen  x4_`filter'_`righthand'_7 = (lag_`filter'_`righthand')^4 *  _b[`righthand'] * L4.`righthand'
  
   replace x1_`filter'_`righthand'_7 = 0 if x1_`filter'_`righthand'_7 == .
  replace x2_`filter'_`righthand'_7 = 0 if x2_`filter'_`righthand'_7 == .
  replace x3_`filter'_`righthand'_7 = 0 if x3_`filter'_`righthand'_7 == .
  replace x4_`filter'_`righthand'_7 = 0 if x4_`filter'_`righthand'_7 == .

  replace xb_`filter'_`righthand'_tsd_7 =  xb_`filter'_`righthand'_tsd_7  + x1_`filter'_`righthand'_7 + x2_`filter'_`righthand'_7 +  x3_`filter'_`righthand'_7 + x4_`filter'_`righthand'_7

 
 reg delta_tfp `righthand'  i.t     [weight=vadd_weight] if (`filter'==1 & year > 1997), vce(cluster sic_group)
  gen xb_`filter'_`righthand'_wt_7 = _b[`righthand'] * `righthand'

  
  outreg2 using "../../Tables/final_`filter'_`righthand'_1997_weighted.tex",   tex(frag) replace nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)


 reg delta_tfp `righthand'   i.t i.sic_group     [weight=vadd_weight] if (`filter'==1 & year > 1997),   vce(cluster sic_group)
  gen xb_`filter'_`righthand'_wts_7 = _b[`righthand'] * `righthand'

  outreg2 using "../../Tables/final_`filter'_`righthand'_1997_weighted.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time")  label symbol(,,,)
 
 xtreg delta_tfp `righthand'   L.delta_tfp  i.t     [weight=vadd_weight] if (`filter'==1 & year > 1997), fe  vce(cluster sic_group)
   gen xb_`filter'_`righthand'_wtsd_7 = _b[`righthand'] * `righthand'
    gen  lag_`filter'_`righthand'_w_7 = _b[L.delta_tfp]
  gen  x1_`filter'_`righthand'_w_7 = lag_`filter'_`righthand' *  _b[`righthand'] * L.`righthand'
  gen  x2_`filter'_`righthand'_w_7 =(lag_`filter'_`righthand')^2 *  _b[`righthand'] * L2.`righthand'
  gen  x3_`filter'_`righthand'_w_7 = (lag_`filter'_`righthand')^3 *  _b[`righthand'] * L3.`righthand'
  gen  x4_`filter'_`righthand'_w_7 = (lag_`filter'_`righthand')^4 *  _b[`righthand'] * L4.`righthand'
  
   replace x1_`filter'_`righthand'_w_7 = 0 if x1_`filter'_`righthand'_w_7 == .
  replace x2_`filter'_`righthand'_w_7 = 0 if x2_`filter'_`righthand'_w_7 == .
  replace x3_`filter'_`righthand'_w_7 = 0 if x3_`filter'_`righthand'_w_7 == .
  replace x4_`filter'_`righthand'_w_7 = 0 if x4_`filter'_`righthand'_w_7 == .

  
   replace xb_`filter'_`righthand'_wtsd_7 =  xb_`filter'_`righthand'_wtsd_7 + x1_`filter'_`righthand'_w_7 + x2_`filter'_`righthand'_w_7 +  x3_`filter'_`righthand'_w_7 + x4_`filter'_`righthand'_w_7

 outreg2 using "../../Tables/final_`filter'_`righthand'_1997_weighted.tex",   tex(frag) append nocons dec(3) nor2  addtext(FE, "Time+SIC")  label symbol(,,,)
}
}


//The next lines aggregate the counterfactual effects of the change in sets by industry over time.
// That is, given that we have computed the predicted counterfactual \beta J_{i,q,t} for each industry and year, we now
// aggregate the effect from 1987-2007. That is, we compute  \sum_{t=1}^{5} \beta J_{i,q,t}  which is the total change in
// industry TFP due to the change in supplier sets aggregated over the entire sample period
foreach variable of varlist xb_*{
egen t`variable' = total(`variable'),by(sic_group)
}


//Export the aggregated data to be used in ../../Code/Matlab/ComputeCounterfactuals.mat
keep if t==5
export  delimited  txb_*   t sic_down using "../../ProcessedData/CSV/Fixed_Effects.csv", replace

